<?php

namespace Drupal\Tests\marketo_ma_webform\Unit;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\marketo_ma\Lead;
use Drupal\marketo_ma\MarketoFieldDefinition;
use Drupal\marketo_ma\Service\MarketoMaServiceInterface;
use Drupal\marketo_ma_webform\Plugin\WebformHandler\MarketoMaWebformHandler;
use Drupal\Tests\UnitTestCase;
use Drupal\webform\WebformSubmissionConditionsValidatorInterface;
use Drupal\webform\WebformSubmissionInterface;
use Symfony\Component\EventDispatcher\EventDispatcher;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * MarketoMaWebformHandler tests.
 *
 * @group marketo_ma_webform
 * @coversDefaultClass \Drupal\marketo_ma_webform\Plugin\WebformHandler\MarketoMaWebformHandler
 */
class MarketoMaWebformHandlerTest extends UnitTestCase {

  /**
   * {@inheritDoc}
   */
  protected function setUp(): void {
    parent::setUp();
    $this->logger_factory = $this->prophesize(LoggerChannelFactoryInterface::class);
    $this->config_factory = $this->prophesize(ConfigFactoryInterface::class);
    $this->entity_type_manager = $this->prophesize(EntityTypeManagerInterface::class);
    $this->conditions_validator = $this->prophesize(WebformSubmissionConditionsValidatorInterface::class);
  }

  /**
   * @covers ::getLead
   */
  public function testGetLeadProf() {
    $webform_submission_array = [
      'my_form_element_1' => 'foo',
      'my_form_element_2' => 'bar',
      'my_form_element_3' => 'baz',
    ];

    $webform_submission = $this->prophesize(WebformSubmissionInterface::class);
    $webform_submission->toArray(TRUE)->willReturn(['data' => $webform_submission_array]);

    $marketo_service = $this->prophesize(MarketoMaServiceInterface::class);
    $marketo_service->getMarketoFields()->willReturn($this->getMarketoFields());

    $sot = new class(
      [
        'settings' => [
          'marketo_ma_mapping' => [
            'my_form_element_1' => 44,
            'my_form_element_2' => 41,
            'my_form_element_3' => 43,
          ],
        ],
      ],
      'marketo_ma',
      '',
      $this->logger_factory->reveal(),
      $this->config_factory->reveal(),
      $this->entity_type_manager->reveal(),
      $this->conditions_validator->reveal(),
      $marketo_service->reveal(),
      new RequestStack(),
      new EventDispatcher()
    ) extends MarketoMaWebformHandler {

      /**
       * {@inheritDoc}
       */
      public function getLead(WebformSubmissionInterface $webform_submission): Lead {
        return parent::getlead($webform_submission);
      }

    };
    $lead = $sot->getLead($webform_submission->reveal());
    $this->assertEquals([
      'email' => 'foo',
      'firstName' => 'bar',
      'lastName' => 'baz',
    ], $lead->data());
  }

  /**
   * Convert raw data to Native field data objects.
   *
   * @return \Drupal\marketo_ma\MarketoFieldDefinition[]
   *   List of mock marketo field definitions.
   */
  private function getMarketoFields() {
    return array_map(function ($field_data) {
      return new MarketoFieldDefinition($field_data);
    }, $this->getMarketoFieldsData());
  }

  /**
   * Get a list of mock marketo fields.
   *
   * @return array[]
   *   Mock marketo field data.
   */
  private function getMarketoFieldsData() {
    return [
      41 => [
        'id' => 41,
        'displayName' => 'First Name',
        'dataType' => 'string',
        'length' => '255',
        'restName' => 'firstName',
        'restReadOnly' => 0,
        'soapName' => 'FirstName',
        'soapReadOnly' => 0,
      ],
      43 => [
        'id' => 43,
        'displayName' => 'Last Name',
        'dataType' => 'string',
        'length' => '255',
        'restName' => 'lastName',
        'restReadOnly' => 0,
        'soapName' => 'LastName',
        'soapReadOnly' => 0,
      ],
      44 => [
        'id' => 44,
        'displayName' => 'Email Address',
        'dataType' => 'email',
        'length' => '255',
        'restName' => 'email',
        'restReadOnly' => 0,
        'soapName' => 'Email',
        'soapReadOnly' => 0,
      ],
      2565 => [
        'id' => 2565,
        'displayName' => 'Additional Comments:',
        'dataType' => 'text',
        'length' => '',
        'restName' => 'Additional_Comments',
        'restReadOnly' => 0,
        'soapName' => 'Additional_Comments',
        'soapReadOnly' => 0,
      ],
    ];
  }

}
